@@ -18,24 +18,27 @@ from whatap.util.hexa32 import Hexa32
 
 import traceback, threading
 
+
 def isEventlet():
     greenlet = sys.modules.get('greenlet')
 
     if greenlet:
         current = greenlet.getcurrent()
         if current is not None and current.parent:
-            print('thread:',threading.get_ident(),'greenlet:', id(current.parent))
+            print('thread:', threading.get_ident(), 'greenlet:', id(current.parent))
             traceback.print_stack()
             return True
     else:
         return False
 
+
 def sendDebugProfile(ctx, msg):
     if ctx:
         ctx.elapsed = 0
         datas = [' ', ' ', 'DEBUG: ' + msg]
         UdpSession.send_packet(PacketTypeEnum.TX_MSG, ctx, datas)
 
+
 def trace_handler(fn, start=False, preload=None):
     def handler(func):
         @wraps(func)
@@ -50,7 +53,7 @@ def trace_handler(fn, start=False, preload=None):
             # print("trace_handler ctx:", ctx)
             if not start and not ctx:
                 return fn(*args, **kwargs)
-            
+
             # check raise step error
             # if ctx and ctx.error_step:
             #     end_interceptor(ctx=ctx)
@@ -59,8 +62,8 @@ def trace_handler(fn, start=False, preload=None):
             try:
                 callback = func(*args, **kwargs)
             except Exception as e:
-                if ctx and ctx.error_step ==e:
-                    ctx.error_step= None
+                if ctx and ctx.error_step == e:
+                    ctx.error_step = None
                     raise e
                 logging.debug(e, extra={'id': 'WA917'}, exc_info=True)
                 print(e, dict(extra={'id': 'WA917'}, exc_info=True))
@@ -71,9 +74,9 @@ def trace_handler(fn, start=False, preload=None):
                     ctx.error_step = None
                     raise e
                 return callback
-        
+
         return wrapper
-    
+
     return handler
 
 
@@ -81,14 +84,14 @@ def start_interceptor(ctx):
     if conf.dev:
         logging.debug('start transaction id(seq): {}'.format(ctx.id),
                       extra={'id': 'WA111'})
-        print('start transaction id(seq): {}'.format(ctx.id),dict(extra={'id': 'WA111'}))
-    
+        print('start transaction id(seq): {}'.format(ctx.id), dict(extra={'id': 'WA111'}))
+
     start_time = DateUtil.nowSystem()
     ctx.start_time = start_time
 
     if ctx.service_name and not ctx.service_hash:
         ctx.service_hash = HashUtil.hashFromString(ctx.service_name)
-    
+
     datas = [ctx.host,
              ctx.service_name,
              ctx.remoteIp,
@@ -99,10 +102,11 @@ def start_interceptor(ctx):
              ]
     # print("start_interceptor txid:", ctx.id)
     UdpSession.send_packet(PacketTypeEnum.TX_START, ctx, datas)
-    
+
     return ctx
 
-def end_interceptor(thread_id=None, ctx= None):
+
+def end_interceptor(thread_id=None, ctx=None):
     if not ctx:
         ctx = TraceContextManager.getContext(
             thread_id) if thread_id else TraceContextManager.getLocalContext()
@@ -113,7 +117,7 @@ def end_interceptor(thread_id=None, ctx= None):
         logging.debug('end   transaction id(seq): {}'.format(ctx.id),
                       extra={'id': 'WA112'})
         print('end   transaction id(seq): {}'.format(ctx.id),
-                      dict(extra={'id': 'WA112'}))
+              dict(extra={'id': 'WA112'}))
 
     start_time = DateUtil.nowSystem()
     ctx.start_time = start_time
@@ -124,28 +128,30 @@ def end_interceptor(thread_id=None, ctx= None):
 
     UdpSession.send_packet(PacketTypeEnum.TX_END, ctx, datas)
 
+
 def parseServiceName(environ):
-    
     return environ.get('PATH_INFO', '')
 
+
 def isIgnore(service_name):
     if conf.trace_ignore_url_prefix and service_name.startswith(conf.trace_ignore_url_prefix):
         return True
 
     if conf.trace_ignore_url_set and service_name in conf.trace_ignore_url_set:
         return True
-    
+
     return False
 
+
 def interceptor(rn_environ, *args, **kwargs):
     if not isinstance(rn_environ, tuple):
         rn_environ = (rn_environ, args[1])
     fn, environ = rn_environ
-    
+
     ctx = TraceContext()
     ctx.host = environ.get('HTTP_HOST', '').split(':')[0]
     ctx.service_name = parseServiceName(environ)
-    
+
     ctx.remoteIp = userid_util.getRemoteAddr(args)
 
     ctx.userAgentString = environ.get('HTTP_USER_AGENT', '')
@@ -159,7 +165,7 @@ def interceptor(rn_environ, *args, **kwargs):
 
     mstt = environ.get('HTTP_{}'.format(
         conf._trace_mtrace_caller_key.upper().replace('-', '_')), '')
-    
+
     if mstt:
         ctx.setTransfer(mstt)
         if conf.stat_mtrace_enabled:
@@ -175,21 +181,21 @@ def interceptor(rn_environ, *args, **kwargs):
             ctx.setTxid(myid)
     caller_poid = environ.get('HTTP_{}'.format(
         conf._trace_mtrace_caller_poid_key.upper().replace('-', '_')), '')
-    
+
     if caller_poid:
         ctx.mcaller_poid = caller_poid
 
     try:
         if isIgnore(ctx.service_name):
             ctx.is_ignored = True
-            return  fn(*args, **kwargs)
+            return fn(*args, **kwargs)
     except Exception as e:
         pass
 
     start_interceptor(ctx)
-    
+
     try:
-        
+
         callback = fn(*args, **kwargs)
         ctx = TraceContextManager.getLocalContext()
         if ctx:
@@ -205,19 +211,19 @@ def interceptor(rn_environ, *args, **kwargs):
                 status_code = callback.status_code
                 errors = [callback.reason_phrase, callback.__class__.__name__]
                 interceptor_error(status_code, errors)
-            
+
             if conf.profile_http_header_enabled:
                 keys = []
                 for key, value in environ.items():
                     if key.startswith('HTTP_'):
                         keys.append(key)
                 keys.sort()
-                
+
                 text = ''
                 for key in keys:
                     text += '{}={}\n'.format(key.split('HTTP_')[1].lower(),
-                                            environ[key])
-                
+                                             environ[key])
+
                 datas = ['HTTP-HEADERS', 'HTTP-HEADERS', text]
                 ctx.start_time = DateUtil.nowSystem()
                 UdpSession.send_packet(PacketTypeEnum.TX_MSG, ctx, datas)
@@ -226,7 +232,6 @@ def interceptor(rn_environ, *args, **kwargs):
         ctx = TraceContextManager.getLocalContext()
         if ctx:
             end_interceptor(ctx=ctx)
-    
 
 
 def interceptor_error(status_code, errors):
@@ -234,12 +239,12 @@ def interceptor_error(status_code, errors):
     ctx.status = int(status_code / 100)
     if ctx.status >= 4 and not ctx.error:
         ctx.error = 1
-        
+
         error = ''
         frame = sys._current_frames().get(ctx.thread.ident)
         if not frame:
             return
-        
+
         for stack in traceback.extract_stack(frame):
             line = stack[0]
             line_num = stack[1]
@@ -249,14 +254,14 @@ def interceptor_error(status_code, errors):
                     '/threading.py') > -1:
                 continue
             error += '{} ({}:{})\n'.format(method_name, line, line_num)
-        
+
         errors.append(error)
-        
+
         # errors.append(''.join(traceback.format_list(traceback.extract_stack(sys._current_frames()[ctx.thread.ident]))))
         UdpSession.send_packet(PacketTypeEnum.TX_ERROR, ctx, errors)
 
 
-def interceptor_step_error(e, ctx = None):
+def interceptor_step_error(e, ctx=None):
     if not ctx:
         ctx = TraceContextManager.getLocalContext()
     if not ctx:
@@ -264,10 +269,14 @@ def interceptor_step_error(e, ctx = None):
     ctx.error_step = e
     if not ctx.error:
         ctx.error = 1
-        
+
     errors = []
     errors.append(e.__class__.__name__)
-    errors.append(str(e.args[0]))
+
+    if e.args:
+        errors.append(str(e.args[0]))
+    elif hasattr(e, "detail"):
+        errors.append(getattr(e, "detail"))
 
     error = ''
     frame = sys._current_frames().get(ctx.thread.ident)
@@ -278,10 +287,10 @@ def interceptor_step_error(e, ctx = None):
         line = stack[0]
         line_num = stack[1]
         method_name = stack[2]
-        
+
         if line.find('/whatap/trace') > -1 or line.find('/threading.py') > -1:
             continue
-        error += '{} ({}:{})\n'.format(method_name,line, line_num)
+        error += '{} ({}:{})\n'.format(method_name, line, line_num)
 
     errors.append(error)
     # errors.append(''.join(traceback.format_list(traceback.extract_stack(sys._current_frames()[ctx.thread.ident]))))
@@ -293,6 +302,7 @@ def interceptor_step_error(e, ctx = None):
         ctx.start_time = DateUtil.nowSystem()
         UdpSession.send_packet(PacketTypeEnum.TX_MSG, ctx, datas)
 
+
 def interceptor_httpc_request(fn, httpc_url, *args, **kwargs):
     ctx = TraceContextManager.getLocalContext()
     if not ctx or ctx.active_httpc_hash:
@@ -316,9 +326,9 @@ def interceptor_httpc_request(fn, httpc_url, *args, **kwargs):
             callback = fn(*args)
             return callback
     except Exception as e:
-        interceptor_step_error(e, ctx = ctx)
+        interceptor_step_error(e, ctx=ctx)
     finally:
-        #ctx = TraceContextManager.getLocalContext()
+        # ctx = TraceContextManager.getLocalContext()
         datas = [ctx.httpc_url, ctx.mcallee]
         ctx.elapsed = DateUtil.nowSystem() - start_time
         UdpSession.send_packet(PacketTypeEnum.TX_HTTPC, ctx, datas)
@@ -336,9 +346,10 @@ def interceptor_httpc_request(fn, httpc_url, *args, **kwargs):
         ctx.active_httpc_hash = 0
         ctx.httpc_url = None
 
+
 def interceptor_sock_connect(fn, *args, **kwargs):
     ctx = TraceContextManager.getLocalContext()
-    if not ctx :
+    if not ctx:
         return fn(*args, **kwargs)
 
     try:
@@ -350,11 +361,12 @@ def interceptor_sock_connect(fn, *args, **kwargs):
     finally:
         ctx.socket_connecting = False
 
+
 def interceptor_db_con(fn, db_type, *args, **kwargs):
     ctx = TraceContextManager.getLocalContext()
     if ctx.db_opening:
         return fn(*args, **kwargs)
-    
+
     start_time = DateUtil.nowSystem()
     ctx.start_time = start_time
 
@@ -377,15 +389,16 @@ def interceptor_db_con(fn, db_type, *args, **kwargs):
     text += kwargs.get('database', kwargs.get('db', kwargs.get('dbname')))
     ctx.active_dbc = text
     ctx.lctx['dbc'] = text
-    
+
     ctx.active_dbc = 0
 
     datas = [text]
     ctx.elapsed = DateUtil.nowSystem() - start_time
     UdpSession.send_packet(PacketTypeEnum.TX_DB_CONN, ctx, datas)
-    
+
     return callback
 
+
 def addQuoteList(arg_dict):
     quoted_dict = dict()
 
@@ -397,20 +410,22 @@ def addQuoteList(arg_dict):
 
     return quoted_dict
 
+
 def addQuoteList(arg_list):
     quoted_list = list()
 
     for v in arg_list:
         if isinstance(v, str):
-            quoted_list.append( "'" + v.replace("'", "\\'") + "'")
+            quoted_list.append("'" + v.replace("'", "\\'") + "'")
         else:
-            quoted_list.append( "'" + str(v) + "'")
+            quoted_list.append("'" + str(v) + "'")
 
     return tuple(quoted_list)
 
+
 def interceptor_db_execute(fn, *args, **kwargs):
     ctx = TraceContextManager.getLocalContext()
-    #sendDebugProfile(ctx, 'interceptor_db_execute step -1')
+    # sendDebugProfile(ctx, 'interceptor_db_execute step -1')
     self = args[0]
     query = None
     if len(args) > 2 and type(args[2]) == dict and args[2]:
@@ -432,7 +447,6 @@ def interceptor_db_execute(fn, *args, **kwargs):
     if not query or ctx.active_sqlhash:
         return fn(*args, **kwargs)
 
-
     start_time = DateUtil.nowSystem()
     ctx.start_time = start_time
     ctx.active_sqlhash = query
@@ -453,21 +467,22 @@ def interceptor_db_execute(fn, *args, **kwargs):
             datas = [' ', ' ', desc]
             ctx.elapsed = 0
             UdpSession.send_packet(PacketTypeEnum.TX_MSG, ctx, datas)
-        
+
         ctx.active_sqlhash = 0
-        
+
+
 def interceptor_db_close(fn, *args, **kwargs):
     ctx = TraceContextManager.getLocalContext()
     ctx.db_opening = False
-    
+
     if not conf.profile_dbc_close:
         return fn(*args, **kwargs)
-    
+
     start_time = DateUtil.nowSystem()
     ctx.start_time = start_time
-    
+
     callback = fn(*args, **kwargs)
-    
+
     text = 'DB: Close Connection.'
     datas = [' ', ' ', text]
     ctx.elapsed = DateUtil.nowSystem() - start_time
@@ -487,16 +502,16 @@ def inter_tx_trace_auto_on(ctx):
     finally:
         if conf.mtrace_rate <= 0 or ctx.httpc_checked or ctx.mtid != 0:
             return
-        
+
         ctx.httpc_checked = True
-        
+
         try:
             inter_tx_trace_auto_on.check_seq += 1
         except AttributeError:
             inter_tx_trace_auto_on.check_seq = 1
         finally:
             check_seq = inter_tx_trace_auto_on.check_seq
-            
+
             rate = int(conf.mtrace_rate / 10)
             if rate == 10:
                 ctx.mtid = TraceContextManager.getId()
